'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

var sharedRoot = {
  fontFamily: 'Menlo, Monaco, "Courier New", monospace',
  fontFeatureSettings: '"liga" 0, "calt" 0',
  lineHeight: '1.5em',
  whiteSpace: 'pre',
  margin: 0
};

var theme = {
  styles: {
    root: /*#__PURE__*/Object.assign({}, sharedRoot, {
      backgroundColor: '#1e2024'
    }),
    attribute: {
      color: '#ef415a'
    },
    unquotedAttribute: {
      color: '#d1d3d7'
    },
    string: {
      color: '#57bedf'
    },
    nil: {
      color: '#f2851e'
    },
    number: {
      color: '#f2851e'
    },
    "boolean": {
      color: '#f2851e'
    },
    punctuation: {
      color: '#d1d3d7'
    }
  }
};

var theme$1 = {
  styles: {
    root: /*#__PURE__*/Object.assign({}, sharedRoot, {
      backgroundColor: '#f8f8f8'
    }),
    attribute: {
      color: '#333'
    },
    unquotedAttribute: {
      color: '#333'
    },
    string: {
      color: '#de1044'
    },
    nil: {
      color: '#008080'
    },
    number: {
      color: '#008080'
    },
    "boolean": {
      color: '#008080'
    },
    punctuation: {
      color: '#333'
    }
  }
};

var theme$2 = {
  styles: {
    root: /*#__PURE__*/Object.assign({}, sharedRoot, {
      backgroundColor: '#2c2d25'
    }),
    attribute: {
      color: '#ddd'
    },
    unquotedAttribute: {
      color: '#ddd'
    },
    string: {
      color: '#a7e12e'
    },
    nil: {
      color: '#fa2572'
    },
    number: {
      color: '#ddd'
    },
    "boolean": {
      color: '#fa2572'
    },
    punctuation: {
      color: '#ddd'
    }
  }
};

var theme$3 = {
  styles: {
    root: /*#__PURE__*/Object.assign({}, sharedRoot, {
      backgroundColor: '#23241e'
    }),
    attribute: {
      color: '#fa2572'
    },
    unquotedAttribute: {
      color: '#fa2572'
    },
    string: {
      color: '#e6db74'
    },
    nil: {
      color: '#ae81ff'
    },
    number: {
      color: '#ae81ff'
    },
    "boolean": {
      color: '#ae81ff'
    },
    punctuation: {
      color: '#f8f8f2'
    }
  }
};

var theme$4 = {
  styles: {
    root: /*#__PURE__*/Object.assign({}, sharedRoot, {
      backgroundColor: '#212121'
    }),
    attribute: {
      color: '#a5e1ff'
    },
    unquotedAttribute: {
      color: '#d9d9d9'
    },
    string: {
      color: '#d49a81'
    },
    nil: {
      color: '#5da8dd'
    },
    number: {
      color: '#bed4b0'
    },
    "boolean": {
      color: '#5da8dd'
    },
    punctuation: {
      color: '#d9d9d9'
    }
  }
};

var theme$5 = {
  styles: {
    root: /*#__PURE__*/Object.assign({}, sharedRoot, {
      backgroundColor: '#fff'
    }),
    attribute: {
      color: '#005db1'
    },
    unquotedAttribute: {
      color: '#000'
    },
    string: {
      color: '#ad0b04'
    },
    nil: {
      color: '#141aff'
    },
    number: {
      color: '#019162'
    },
    "boolean": {
      color: '#141aff'
    },
    punctuation: {
      color: '#000'
    }
  }
};



var allThemes = {
  __proto__: null,
  anOldHopeClassic: theme,
  github: theme$1,
  monokai: theme$2,
  monokaiSublime: theme$3,
  vscodeDark: theme$4,
  vscodeLight: theme$5
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any

var JasonContext = /*#__PURE__*/React.createContext({});
function useTokenMachine() {
  var _useContext = React.useContext(JasonContext),
      token = _useContext.token;

  var _char = React.useCallback(function (sym) {
    return token(sym === '"' || sym === "'" ? 'quotation' : 'punctuation', sym);
  }, [token]);

  return {
    token: token,
    "char": _char
  };
}
function useToken(token, children, as) {
  if (as === void 0) {
    as = 'span';
  }

  return useTokenMachine().token(token, children, as);
}
function createTokenMachine(theme) {
  return function tokenMachine(token, children, as) {
    if (as === void 0) {
      as = 'span';
    }

    var style = theme.styles ? theme.styles[token] : undefined;
    var className = theme.classes ? theme.classes[token] : undefined;
    var styled = Boolean(style || className);
    return styled ? React.createElement(as, {
      style: style,
      className: className
    }, children) : React__default.createElement(React__default.Fragment, null, children);
  };
}

var defaultItemKeyGenerator = function defaultItemKeyGenerator( // @ts-ignore
item, basePath, index) {
  return basePath + "[" + index + "]";
};

var themes = allThemes;
var ReactJason = function ReactJason(_ref) {
  var value = _ref.value,
      theme$1 = _ref.theme,
      nodeWrapper = _ref.nodeWrapper,
      itemKeyGenerator = _ref.itemKeyGenerator,
      _ref$quoteAttributes = _ref.quoteAttributes,
      quoteAttributes = _ref$quoteAttributes === void 0 ? true : _ref$quoteAttributes;
  var token = React__default.useMemo(function () {
    return createTokenMachine(theme$1 || theme);
  }, [theme$1]);
  var getItemKey = itemKeyGenerator || defaultItemKeyGenerator;
  var context = React__default.useMemo(function () {
    return {
      token: token,
      getItemKey: getItemKey,
      quoteAttributes: quoteAttributes,
      nodeWrapper: nodeWrapper
    };
  }, [token, getItemKey, quoteAttributes, nodeWrapper]);
  return React__default.createElement(JasonContext.Provider, {
    value: context
  }, token('root', node({
    value: value,
    path: '',
    depth: 1,
    nodeWrapper: nodeWrapper
  }), 'pre'));
};

function node(_ref2) {
  var value = _ref2.value,
      path = _ref2.path,
      depth = _ref2.depth,
      NodeWrapper = _ref2.nodeWrapper;

  var wrap = function wrap(children, type) {
    return NodeWrapper ? React__default.createElement(NodeWrapper, {
      path: path,
      type: type
    }, children) : children;
  };

  if (value === null) {
    return wrap(React__default.createElement(NullNode, null), 'nil');
  }

  if (Array.isArray(value)) {
    return wrap(React__default.createElement(ArrayNode, {
      value: value,
      path: path,
      depth: depth
    }), 'array');
  }

  switch (typeof value) {
    case 'string':
      return wrap(React__default.createElement(StringNode, {
        value: value,
        path: path,
        depth: depth
      }), 'string');

    case 'number':
      return wrap(React__default.createElement(NumberNode, {
        value: value,
        path: path,
        depth: depth
      }), 'number');

    case 'boolean':
      return wrap(React__default.createElement(BooleanNode, {
        value: value,
        path: path,
        depth: depth
      }), 'boolean');

    case 'object':
      return wrap(React__default.createElement(ObjectNode, {
        value: value,
        path: path,
        depth: depth
      }), 'object');

    default:
      throw new Error("Unhandled type " + typeof value);
  }
}

function StringNode(_ref3) {
  var value = _ref3.value;

  var _useTokenMachine = useTokenMachine(),
      _char = _useTokenMachine["char"];

  var unquoted = json(value).slice(1, -1);
  return useToken('string', React__default.createElement(React__default.Fragment, null, _char('"'), unquoted, _char('"')));
}

function NumberNode(_ref4) {
  var value = _ref4.value;
  return useToken('number', value);
}

function BooleanNode(_ref5) {
  var value = _ref5.value;
  return useToken('boolean', value ? 'true' : 'false');
}

function ArrayNode(_ref6) {
  var value = _ref6.value,
      path = _ref6.path,
      depth = _ref6.depth;

  var _useTokenMachine2 = useTokenMachine(),
      token = _useTokenMachine2.token,
      _char2 = _useTokenMachine2["char"];

  var _useContext = React.useContext(JasonContext),
      getItemKey = _useContext.getItemKey,
      nodeWrapper = _useContext.nodeWrapper;

  var numItems = value.length;
  return token('array', React__default.createElement(React__default.Fragment, null, _char2('['), "\n" + indent(depth), value.map(function (item, index) {
    var itemPath = getItemKey(item, path, index);
    var isLast = index === numItems - 1;
    return React__default.createElement(React.Fragment, {
      key: itemPath
    }, node({
      value: item,
      path: itemPath,
      depth: depth + 1,
      nodeWrapper: nodeWrapper
    }), isLast ? "\n" + indent(depth - 1) : React__default.createElement(React__default.Fragment, null, _char2(','), "\n" + indent(depth)));
  }), _char2(']')));
}

function ObjectNode(_ref7) {
  var obj = _ref7.value,
      path = _ref7.path,
      depth = _ref7.depth;

  var _useContext2 = React.useContext(JasonContext),
      nodeWrapper = _useContext2.nodeWrapper;

  var _useTokenMachine3 = useTokenMachine(),
      token = _useTokenMachine3.token,
      _char3 = _useTokenMachine3["char"];

  var value = obj;
  var keys = Object.keys(value);
  var lastKey = keys.length - 1;
  return token('object', React__default.createElement(React__default.Fragment, null, _char3('{'), '\n', keys.map(function (key, index) {
    var val = value[key];
    var propPath = path ? path + "." + key : key;
    return React__default.createElement(React.Fragment, {
      key: propPath
    }, React__default.createElement(AttributePair, {
      attribute: key,
      value: val,
      depth: depth,
      path: propPath,
      nodeWrapper: nodeWrapper,
      isLastKey: index === lastKey
    }));
  }), indent(depth - 1), _char3('}')));
}

function AttributeNode(_ref8) {
  var value = _ref8.value;

  var _useContext3 = React.useContext(JasonContext),
      quoteAttributes = _useContext3.quoteAttributes;

  var _useTokenMachine4 = useTokenMachine(),
      token = _useTokenMachine4.token,
      _char4 = _useTokenMachine4["char"];

  var attributeName = json(value).slice(1, -1);
  var attribute;

  if (quoteAttributes || attributeName.includes('"')) {
    attribute = token('attribute', React__default.createElement(React__default.Fragment, null, _char4('"'), attributeName, _char4('"')));
  } else {
    attribute = token('unquotedAttribute', attributeName);
  }

  return React__default.createElement(React__default.Fragment, null, attribute, _char4(':'), ' ');
}

function AttributePair(_ref9) {
  var attribute = _ref9.attribute,
      value = _ref9.value,
      path = _ref9.path,
      depth = _ref9.depth,
      isLastKey = _ref9.isLastKey,
      NodeWrapper = _ref9.nodeWrapper;

  var _useTokenMachine5 = useTokenMachine(),
      _char5 = _useTokenMachine5["char"];

  var pair = React__default.createElement(React.Fragment, {
    key: path
  }, indent(depth), React__default.createElement(AttributeNode, {
    value: attribute
  }), node({
    value: value,
    path: path,
    depth: depth + 1,
    nodeWrapper: NodeWrapper
  }), isLastKey ? '\n' : React__default.createElement(React__default.Fragment, null, _char5(','), '\n'));
  return NodeWrapper ? React__default.createElement(NodeWrapper, {
    type: "attributePair",
    path: path
  }, pair) : pair;
}

function NullNode() {
  return useToken('nil', 'null');
}

function json(value) {
  return JSON.stringify(value);
}

function indent(depth) {
  return '  '.repeat(depth);
}

exports.ReactJason = ReactJason;
exports.themes = themes;
//# sourceMappingURL=react-jason.cjs.development.js.map
